<?php
require_once("include.php");

/*
 * generate a unique filename for the given directory and base file name - we use
 * this to make sure that thumbnails and other uploaded files are always unique, and
 * don't get overwritten.
 */
function unique_file_name( $basedir, $basename ) {

	$ext = ereg_replace("^.+\\.([^.]+)$", "\\1", $basename );
	$hashedname = md5($basename . rand() ) . "." . $ext;

	while ( file_exists($basedir . "/" . $hashedname ) ) {
		$hashedname = md5($basename . rand() ) . "." . $ext;
	}

	return $hashedname;
}

function is_valid_mimetype($type = "text/html") {

	if ( beginsWith($type, "video/") ||
		beginsWith($type, "audio/") ||
		$type == "application/x-bittorrent" ) {
		return true;
	}

	return false;
}

/*
 * publish a file from POST input
 */
function publish_file($file) {

	//
	// if the user doesn't have upload access, then stop right here
	//
	requireUploadAccess();

	global $store;
	global $errorstr;
	global $perm_level;

	foreach ($file as $itemName => $item) {
		if ( $itemName != "post_channels" ) {
			$file[$itemName] = stripslashes($item);
		}
	}

	//
	// grab our incoming variables here
	//	
	$file_url = $file["post_file_url"];
	
	$title = encode($file["post_title"]);
	$desc = encode($file["post_desc"]);
	$image = $file["post_image"];
	$license_url = encode($file["post_license_url"]);
	$licenseName = encode($file["post_license_name"]);
	$creator = encode($file["post_creator"]);
	$rights = encode($file["post_rights"]);
	$temp_keywords = explode("\n", $file["post_keywords"]);
	$people_array = explode("\n", $file["post_people"]);
	$webpage = encode($file["post_webpage"]);
	$mimetype = $file["post_mimetype"];
	$runtime_hours = $file["post_length_hours"];
	$runtime_minutes = $file["post_length_minutes"];
	$runtime_seconds = $file["post_length_seconds"];
	$publish_date = strtotime(($file["post_publish_month"] + 1) . "/" . $file["post_publish_day"] . "/" . $file["post_publish_year"] . " " . $file["post_publish_hour"] . ":" . $file["post_publish_minute"]);

	// paranoia check here - make sure we always have a good time value
	if ( $publish_date <= 0 ) {
		$publish_date = time();
	}
	
	$release_year = $file["post_release_year"];
	$release_month = $file["post_release_month"];
	$release_day = $file["post_release_day"];
//	$channelIDs = explode(",", $file["post_channel_array"]);

	if ( isset($file["post_channels"]) && count($file["post_channels"]) > 0 ) {
		foreach($file["post_channels"] as $tmp) {
			$channelIDs[$tmp] = $tmp;	
		}
	}
	else {
		$channelIDs = array();
	}

	$got_mime_type = false;
	
		// if this is a torrent posted with the helper, then we'll have a hash already
	$filehash = $file["post_filehash"];
	//
	// generate the hash which will be used to identify this file
	//
	if ($filehash == "") {
		$filehash = sha1($file_url);
	}

	if ( isset($file["post_mimetype"]) && $file["post_mimetype"] == "application/x-bittorrent" ) {
		$got_mime_type = true;
		$mimetype = "application/x-bittorrent";
	}

	//
	// this is set if the user is uploading a file using http upload
	//
	else if (isset($_FILES["post_file_upload"]) && $_FILES["post_file_upload"]["size"] > 0 ) {

		if (!file_exists('torrents')) {
			mkdir("torrents",$perm_level);
		}

		$fname = unique_file_name("torrents", $_FILES['post_file_upload']['name']);
				
		if (
			move_uploaded_file(
				$_FILES['post_file_upload']['tmp_name'], 
				"torrents/" . $fname ) ) {

			chmod("torrents/" . $fname, $perm_level);
			$file_url = get_base_url() . "torrents/" . $fname;
		}

		if ( isset($_FILES["post_file_upload"]["type"]) && 
				$_FILES["post_file_upload"]["type"] != "" && 
				is_valid_mimetype($_FILES["post_file_upload"]["type"]) ) {
			$mimetype = $_FILES["post_file_upload"]["type"];
			$got_mime_type = true;
		}
		else {
			$mimetype = @mime_content_type("torrents/" . $fname);
			if ( $mimetype ) {
				$got_mime_type = true;
			}
		}

	}

	//
	// if we've got a URL here, let's try and figure out the content-type
	//
//	if ( $got_mime_type == false && strpos(strtolower($file_url), "http://") === 0 ) {
	if ( $got_mime_type == false ) {

		$errstr = "";
		$mimetype = get_content_type($file_url, $errstr);

		// we got an error, set our global error variable and exit out
		if ( $errstr ) {
			$errorstr = $errstr;
			return false;
		}
	}
//	else if ( ! $file["post_filehash"] && $_FILES["post_file_upload"]["size"] <= 0 ) {
		//$errorstr = "Please post a file";
//		return false;
//	}

	//
	// we'll share this file if the checkbox was checked, and it happens to be
	// a local torrent file
	//
	$sharing_enabled = isset($file["sharing_enabled"]) &&
							(
								(isset($filehash) && $filehash != "") ||
								(isset($file["post_file_url"]) && is_local_torrent($file["post_file_url"]))
							);



	if ( ! file_exists('publish') ) {
		mkdir("publish", $perm_level);
	}
	
	if ( isset($channelIDs) && count($channelIDs) > 0 ) {
		foreach ($channelIDs as $channelID) {
			if(!file_exists("publish/" . $channelID)) {
				mkdir("publish/" . $channelID, $perm_level);
			}
		
			$handle = fopen("publish/" . $channelID . "/" . $publish_date, "a+b");
			fclose($handle);
		}
	}	
	
	$explicit = 0;
	
	if (isset($file["post_explicit"])) {
		$explicit = 1;
	}
	
	$excerpt = 0;
	
	if (isset($file["post_is_excerpt"])) {
		$excerpt = 1;
	}


	$transcript_url = $file["post_transcript_url"];

	if (isset($_FILES["post_transcript_file"])) {
		if ($_FILES["post_transcript_file"]["size"] > 0) {
			if (!file_exists('text')) {
				mkdir("text",$perm_level);
			}
	
			if (move_uploaded_file($_FILES['post_transcript_file']['tmp_name'], "text/" . $filehash . ".txt")) {
				chmod("text/" . $filehash, 0644);
				$transcript_url = get_base_url() . "text/" . $filehash . ".txt";
			}
		}
	}


	if ($file["post_transcript_text"]) {
		if (!file_exists('text')) {
			mkdir("text");
		}
	
		$handle = fopen('text/' . $filehash . '.txt', "a+b");
		fseek($handle,0);
		flock($handle,LOCK_EX);
		ftruncate($handle,0);
		fseek($handle,0);
		fwrite($handle,$file["post_transcript_text"]);
		fclose($handle);
	
		$transcript_url = get_base_url() . "text/" . $filehash . ".txt";
	}
	
	//
	// handle any thumbnail that the user posted
	//
	if (isset($_FILES["post_image_upload"])) {
		if (!file_exists('thumbnails')) {
			mkdir("thumbnails",$perm_level);
		}

		$hashedname = unique_file_name("thumbnails", $_FILES['post_image_upload']['name']);
	
		if (
			move_uploaded_file(
				$_FILES['post_image_upload']['tmp_name'], 
				"thumbnails/" . $hashedname ) ) {
	
			chmod("thumbnails/" . $hashedname, 0644);
			$image = get_base_url() . "thumbnails/" . $hashedname;
		}
	
	}

	//
	// parse keywords
	//
	$keywords = array();
	
	foreach ($temp_keywords as $words) {
		if (trim($words) != '') {
			$keywords[] = encode(trim($words));
		}
	}

	//
	// parse people
	//
	$people = array();
	
	foreach ($people_array as $people_row) {
		if (trim($people_row) != '') {
			$people[] = explode(":", encode($people_row));
		}
	}
	
	
	if ($image == "http://") {
		$image = '';
	}

	//
	// lets figure out if we have a local file or a remote URL here.
	// if it's a file, then we will also check and see if it's a torrent
	//	
	if ( file_exists("data/" . $file_url ) ) {

		// data/$file_url will contain the name of the torrent
// cjm - use binary mode?
//		$handle = fopen('data/' . $file_url, "r+");
		$handle = fopen('data/' . $file_url, "r+");
		if ( $handle ) {
			$torrent = fread($handle, 1024);
			fclose($handle);
			$file_url = get_base_url() . 'torrents/' . $torrent;
		}
	}

	//
	// create a new file entry, load in our data, and save it
	//
	$newcontent = $store->read_files();

	$newcontent[$filehash]['URL'] = $file_url;
	$newcontent[$filehash]['Title'] = $title;
	$newcontent[$filehash]['Desc'] = $desc;
	$newcontent[$filehash]['Image'] = $image;
	$newcontent[$filehash]['LicenseURL'] = $license_url;
	$newcontent[$filehash]['LicenseName'] = $licenseName;
	$newcontent[$filehash]['Creator'] = $creator;
	$newcontent[$filehash]['Rights'] = $rights;
	$newcontent[$filehash]['Keywords'] = $keywords;
	$newcontent[$filehash]['People'] = $people;
	$newcontent[$filehash]['Webpage'] = $webpage;
	$newcontent[$filehash]['Mimetype'] = trim($mimetype);
	$newcontent[$filehash]['RuntimeHours'] = $runtime_hours;
	$newcontent[$filehash]['RuntimeMinutes'] = $runtime_minutes;
	$newcontent[$filehash]['RuntimeSeconds'] = $runtime_seconds;
	$newcontent[$filehash]['Publishdate'] = $publish_date;
	$newcontent[$filehash]['ReleaseYear'] = $release_year;
	$newcontent[$filehash]['ReleaseMonth'] = $release_month;
	$newcontent[$filehash]['ReleaseDay'] = $release_day;
	$newcontent[$filehash]['Transcript'] = $transcript_url;
	$newcontent[$filehash]['Explicit'] = $explicit;
	$newcontent[$filehash]['Excerpt'] = $excerpt;
	$newcontent[$filehash]['Created'] = time();
	
	$newcontent[$filehash]['SharingEnabled'] = $sharing_enabled;

	if (!isset($newcontent[$filehash]['Publisher'])) {
		if (isset($_SESSION['user']['Name'])) {
			$newcontent[$filehash]['Publisher'] = $_SESSION['user']['Name'];
		}
	}
	
	$store->store_files($newcontent);



	//
	// write out any channel info
	//
	$channels = $store->getAllChannels();

	foreach ($channels as $channel) {
		if (is_admin() || $channel["OpenPublish"]) {
			$keys = array_keys($channel['Files']);

			//
			// first, unset any channels that this was published to
			//
			foreach ($keys as $key) {
				$file = $channel['Files'][$key];
				if ($file[0] == $filehash) {
					unset($channel['Files'][$key]);
				}
			}
	

			if ( isset($channelIDs) && count($channelIDs) > 0 &&
				in_array($channel['ID'], $channelIDs) ) {

				$sections = array_keys($channel['Sections']);
	
				foreach ($sections as $section) {
					$keys = array_keys($channel['Sections'][$section]['Files']);
	
					foreach ($keys as $key) {
						$file = $channel['Sections'][$section]['Files'][$key];
						if ($file == $filehash) {
							unset($channel['Sections'][$section]['Files'][$key]);
						}
					}
				}
			}
	
			$channels[$channel['ID']] = $channel;
	
		}
	}

	$store->store_channels($channels);
	$channels = $store->getAllChannels();


	if ( isset($channelIDs) && count($channelIDs) > 0 ) {
		foreach ($channelIDs as $channelID) {
		
			if ($channelID != '') {
				if (is_admin() || $channels[$channelID]["OpenPublish"]) {
					$channels[$channelID]["Files"][] = array($filehash, $publish_date);
				}
			}
		}
		
		$store->store_channels($channels);
	
		//
		// generate new RSS feeds for any channels we just re-published
		//
		$channels = $store->getAllChannels();
		foreach ($channelIDs as $channelID) {
			makeChannelRss($channelID);
		}
	}

	global $seeder;
	global $settings;

	//
	// if this is a torrent, and we're configured to share all torrents, then
	// start sharing it
	//
	if ( 
		// is a local torrent
		is_local_torrent($file_url) && 

		// sharing is turned on
		isset($settings['sharing_enable']) && $settings['sharing_enable'] == 1 &&

		(
			// it's shared - OR - 
			$sharing_enabled == true ||

			// global sharing is on		
			(isset($settings['sharing_auto']) && $settings['sharing_auto'] == 1)
		)
	) {
//		$torrentfile = urldecode(substr($file_url,strlen(get_base_url()) + 9));
		$torrentfile = local_filename($file_url);
		$seeder->spawn($torrentfile);
	}

	return true;
}
?>